package org.openxma.dsl.generator;

import static com.google.inject.Key.get;
import static com.google.inject.name.Names.named;

import org.eclipse.emf.common.util.URI;
import org.eclipse.xtext.resource.IResourceServiceProvider;

import com.google.inject.Injector;
import com.google.inject.Module;

/**
 * Default factory which creates generator instances based on property values
 * retrieved from a given GeneratorConfiguration.
 */
public class DefaultGeneratorFactory implements GeneratorFactory {

	public Generator createAndInit(GeneratorConfiguration configuration) {
		Generator result = null;
		Module module = loadGeneratorModule(configuration);
		if (module == null) {
			result = createAndInitXpandGenerator(configuration);
		} else {
			String fileExtension = configuration.getFileExtension();
			try {
				IResourceServiceProvider resourceServiceProvider = IResourceServiceProvider.Registry.INSTANCE
						.getResourceServiceProvider(URI.createURI("null")
								.appendFileExtension(fileExtension));
				Injector injector = resourceServiceProvider.get(Injector.class);
				Injector generatorInjector = injector
						.createChildInjector(module);
				result = generatorInjector.getInstance(get(Generator.class,
						named(fileExtension + "Generator")));
			} catch (Exception classNotFoundException) {
				result = createAndInitXpandGenerator(configuration);
			}
		}

		if (result != null) {
			result.init(configuration);
		}
		return result;

	}

	protected Module loadGeneratorModule(GeneratorConfiguration configuration) {
		String moduleClass = configuration.getProperty("generator.module");
		if (moduleClass == null) {
			return null;
		}
		return DefaultGeneratorConfiguration.newInstanceOf(moduleClass);
	}

	protected Generator createAndInitXpandGenerator(
			GeneratorConfiguration generatorConfiguration) {
		String generatorClassName = generatorConfiguration.getProperty(String
				.format("%s.generator",
						generatorConfiguration.getFileExtension()));
		return DefaultGeneratorConfiguration.newInstanceOf(generatorClassName);
	}

}
